@@ -55,24 +55,6 @@ import org.apache.commons.logging.Log;
  * @since 3.0
  */
 public abstract class InterpreterBase extends ParserVisitor {
-    /**
-     * Cached arithmetic function call.
-     */
-    protected static class ArithmeticFuncall extends Funcall {
-        /**
-         * Constructs a new instance.
-         * @param jme  the method
-         * @param flag the narrow flag
-         */
-        protected ArithmeticFuncall(final JexlMethod jme, final boolean flag) {
-            super(jme, flag);
-        }
-
-        @Override
-        protected Object tryInvoke(final InterpreterBase ii, final String name, final Object target, final Object[] args) {
-            return me.tryInvoke(name, ii.arithmetic, ii.functionArguments(target, narrow, args));
-        }
-    }
     /**
      * Helping dispatch function calls.
      */
@@ -208,6 +190,26 @@ public abstract class InterpreterBase extends ParserVisitor {
             return JexlEngine.TRY_FAILED;
         }
     }
+
+    /**
+     * Cached arithmetic function call.
+     */
+    protected static class ArithmeticFuncall extends Funcall {
+        /**
+         * Constructs a new instance.
+         * @param jme  the method
+         * @param flag the narrow flag
+         */
+        protected ArithmeticFuncall(final JexlMethod jme, final boolean flag) {
+            super(jme, flag);
+        }
+
+        @Override
+        protected Object tryInvoke(final InterpreterBase ii, final String name, final Object target, final Object[] args) {
+            return me.tryInvoke(name, ii.arithmetic, ii.functionArguments(target, narrow, args));
+        }
+    }
+
     /**
      * Cached context function call.
      */
@@ -274,8 +276,10 @@ public abstract class InterpreterBase extends ParserVisitor {
             return me.tryInvoke(name, target, ii.functionArguments(null, narrow, args));
         }
     }
+
     /** Empty parameters for method matching. */
     protected static final Object[] EMPTY_PARAMS = {};
+
     /**
      * Pretty-prints a failing property value (de)reference.
      * <p>Used by calls to unsolvableProperty(...).</p>
@@ -285,6 +289,7 @@ public abstract class InterpreterBase extends ParserVisitor {
     protected static String stringifyPropertyValue(final JexlNode node) {
         return node != null ? new Debugger().depth(1).data(node) : "???";
     }
+
     /** The JEXL engine. */
     protected final Engine jexl;
     /** The logger. */
@@ -299,22 +304,16 @@ public abstract class InterpreterBase extends ParserVisitor {
     protected final JexlOptions options;
     /** Cache executors. */
     protected final boolean cache;
-
     /** Cancellation support. */
     protected final AtomicBoolean cancelled;
-
     /** The namespace resolver. */
     protected final JexlContext.NamespaceResolver ns;
-
     /** The class name resolver. */
     protected final JexlContext.ClassNameResolver fqcnSolver;
-
     /** The operators evaluation delegate. */
-    protected final Operators operators;
-
+    protected final JexlOperator.Uberspect operators;
     /** The map of 'prefix:function' to object resolving as namespaces. */
     protected final Map<String, Object> functions;
-
     /** The map of dynamically created namespaces, NamespaceFunctor or duck-types of those. */
     protected Map<String, Object> functors;
 
@@ -350,7 +349,11 @@ public abstract class InterpreterBase extends ParserVisitor {
         this.cancelled = acancel != null ? acancel : new AtomicBoolean();
         this.functions = options.getNamespaces();
         this.functors = null;
-        this.operators = new Operators(this);
+        JexlOperator.Uberspect ops = uberspect.getOperator(arithmetic);
+        if (ops == null) {
+            ops = new Operator(uberspect, arithmetic);
+        }
+        this.operators = ops;
         // the import package facility
         final Collection<String> imports = options.getImports();
         this.fqcnSolver = imports.isEmpty()
@@ -468,28 +471,28 @@ public abstract class InterpreterBase extends ParserVisitor {
     /**
      * Triggered when a captured variable is const and assignment is attempted.
      * @param node  the node where the error originated from
-     * @param var   the variable name
+     * @param variable   the variable name
      * @return throws JexlException if strict and not silent, null otherwise
      */
-    protected Object constVariable(final JexlNode node, final String var) {
-        return variableError(node, var, VariableIssue.CONST);
+    protected Object constVariable(final JexlNode node, final String variable) {
+        return variableError(node, variable, VariableIssue.CONST);
     }
 
     /**
      * Defines a variable.
-     * @param var the variable to define
+     * @param variable the variable to define
      * @param frame the frame in which it will be defined
      * @return true if definition succeeded, false otherwise
      */
-    protected boolean defineVariable(final ASTVar var, final LexicalFrame frame) {
-        final int symbol = var.getSymbol();
+    protected boolean defineVariable(final ASTVar variable, final LexicalFrame frame) {
+        final int symbol = variable.getSymbol();
         if (symbol < 0) {
             return false;
         }
-        if (var.isRedefined()) {
+        if (variable.isRedefined()) {
             return false;
         }
-        return frame.defineSymbol(symbol, var.isCaptured());
+        return frame.defineSymbol(symbol, variable.isCaptured());
     }
 
     /**
@@ -509,6 +512,9 @@ public abstract class InterpreterBase extends ParserVisitor {
         return node;
     }
 
+    /**
+     * @deprecated
+     */
     @Deprecated
     protected JexlNode findNullOperand(final RuntimeException xrt, final JexlNode node, final Object left, final Object right) {
         return findNullOperand(node, left, right);
@@ -690,7 +696,7 @@ public abstract class InterpreterBase extends ParserVisitor {
      * @return true if cancelled, false otherwise
      */
     protected boolean isCancelled() {
-        return cancelled.get() | Thread.currentThread().isInterrupted();
+        return cancelled.get() || Thread.currentThread().isInterrupted();
     }
 
     /**
@@ -774,7 +780,7 @@ public abstract class InterpreterBase extends ParserVisitor {
     /**
      * Triggered when an operator fails.
      * @param node     the node where the error originated from
-     * @param operator the method name
+     * @param operator the operator symbol
      * @param cause    the cause of error (if any)
      * @return throws JexlException if strict and not silent, null otherwise
      */
@@ -791,11 +797,11 @@ public abstract class InterpreterBase extends ParserVisitor {
     /**
      * Triggered when a variable is lexically known as being redefined.
      * @param node  the node where the error originated from
-     * @param var   the variable name
+     * @param variable   the variable name
      * @return throws JexlException if strict and not silent, null otherwise
      */
-    protected Object redefinedVariable(final JexlNode node, final String var) {
-        return variableError(node, var, VariableIssue.REDEFINED);
+    protected Object redefinedVariable(final JexlNode node, final String variable) {
+        return variableError(node, variable, VariableIssue.REDEFINED);
     }
 
     /**
@@ -863,11 +869,11 @@ public abstract class InterpreterBase extends ParserVisitor {
                             if (functor != null) {
                                 // wrap the namespace in a NamespaceFunctor to shield us from the actual
                                 // number of arguments to call it with.
-                                final Object ns = namespace;
+                                final Object nsFinal = namespace;
                                 // make it a class (not a lambda!) so instanceof (see *2) will catch it
                                 namespace = (NamespaceFunctor) context -> withContext
-                                        ? ctor.tryInvoke(null, ns, context)
-                                        : ctor.tryInvoke(null, ns);
+                                        ? ctor.tryInvoke(null, nsFinal, context)
+                                        : ctor.tryInvoke(null, nsFinal);
                                 if (cacheable && ctor.isCacheable()) {
                                     nsNode.jjtSetValue(namespace);
                                 }
@@ -1027,15 +1033,15 @@ public abstract class InterpreterBase extends ParserVisitor {
     /**
      * Triggered when a variable is lexically known as undefined.
      * @param node  the node where the error originated from
-     * @param var   the variable name
+     * @param variable   the variable name
      * @return throws JexlException if strict and not silent, null otherwise
      */
-    protected Object undefinedVariable(final JexlNode node, final String var) {
-        return variableError(node, var, VariableIssue.UNDEFINED);
+    protected Object undefinedVariable(final JexlNode node, final String variable) {
+        return variableError(node, variable, VariableIssue.UNDEFINED);
     }
 
     /**
-     * Triggered when a method can not be resolved.
+     * Triggered when a method cannot be resolved.
      * @param node   the node where the error originated from
      * @param method the method name
      * @return throws JexlException if strict and not silent, null otherwise
@@ -1045,7 +1051,7 @@ public abstract class InterpreterBase extends ParserVisitor {
     }
 
     /**
-     * Triggered when a method can not be resolved.
+     * Triggered when a method cannot be resolved.
      * @param node   the node where the error originated from
      * @param method the method name
      * @param args the method arguments
@@ -1062,7 +1068,7 @@ public abstract class InterpreterBase extends ParserVisitor {
     }
 
     /**
-     * Triggered when a property can not be resolved.
+     * Triggered when a property cannot be resolved.
      * @param node  the node where the error originated from
      * @param property   the property node
      * @param cause the cause if any
@@ -1080,29 +1086,29 @@ public abstract class InterpreterBase extends ParserVisitor {
     }
 
     /**
-     * Triggered when a variable can not be resolved.
+     * Triggered when a variable cannot be resolved.
      * @param node  the node where the error originated from
-     * @param var   the variable name
+     * @param variable   the variable name
      * @param undef whether the variable is undefined or null
      * @return throws JexlException if strict and not silent, null otherwise
      */
-    protected Object unsolvableVariable(final JexlNode node, final String var, final boolean undef) {
-        return variableError(node, var, undef? VariableIssue.UNDEFINED : VariableIssue.NULLVALUE);
+    protected Object unsolvableVariable(final JexlNode node, final String variable, final boolean undef) {
+        return variableError(node, variable, undef? VariableIssue.UNDEFINED : VariableIssue.NULLVALUE);
     }
 
     /**
      * Triggered when a variable generates an issue.
      * @param node  the node where the error originated from
-     * @param var   the variable name
+     * @param variable   the variable name
      * @param issue the issue type
      * @return throws JexlException if strict and not silent, null otherwise
      */
-    protected Object variableError(final JexlNode node, final String var, final VariableIssue issue) {
+    protected Object variableError(final JexlNode node, final String variable, final VariableIssue issue) {
         if (isStrictEngine() && !isTernaryProtected(node)) {
-            throw new JexlException.Variable(node, var, issue);
+            throw new JexlException.Variable(node, variable, issue);
         }
         if (logger.isDebugEnabled()) {
-            logger.debug(JexlException.variableError(node, var, issue));
+            logger.debug(JexlException.variableError(node, variable, issue));
         }
         return null;
     }
