@@ -21,13 +21,8 @@ package org.apache.commons.jexl3.parser;
  */
 public class ASTIdentifier extends JexlNode {
     /**
-     *
      */
     private static final long serialVersionUID = 1L;
-    protected String name = null;
-    protected int symbol = -1;
-    protected int flags = 0;
-
     /** The redefined variable flag. */
     private static final int REDEFINED = 0;
     /** The shaded variable flag. */
@@ -38,47 +33,6 @@ public class ASTIdentifier extends JexlNode {
     private static final int LEXICAL = 3;
     /** The const variable flag. */
     private static final int CONST = 4;
-
-    ASTIdentifier(final int id) {
-        super(id);
-    }
-
-    ASTIdentifier(final Parser p, final int id) {
-        super(p, id);
-    }
-
-    @Override
-    public String toString() {
-        return name;
-    }
-
-    void setSymbol(final String identifier) {
-        if (identifier.charAt(0) == '#') {
-            symbol = Integer.parseInt(identifier.substring(1));
-        }
-        name = identifier;
-    }
-
-    void setSymbol(final int r, final String identifier) {
-        symbol = r;
-        name = identifier;
-    }
-
-    public int getSymbol() {
-        return symbol;
-    }
-
-    /**
-     * Sets the value of a flag in a mask.
-     * @param ordinal the flag ordinal
-     * @param mask the flags mask
-     * @param value true or false
-     * @return the new flags mask value
-     */
-    private static int set(final int ordinal, final int mask, final boolean value) {
-        return value? mask | (1 << ordinal) : mask & ~(1 << ordinal);
-    }
-
     /**
      * Checks the value of a flag in the mask.
      * @param ordinal the flag ordinal
@@ -88,46 +42,24 @@ public class ASTIdentifier extends JexlNode {
     private static boolean isSet(final int ordinal, final int mask) {
         return (mask & 1 << ordinal) != 0;
     }
-
-    public void setRedefined(final boolean f) {
-        flags = set(REDEFINED, flags, f);
+    /**
+     * Sets the value of a flag in a mask.
+     * @param ordinal the flag ordinal
+     * @param mask the flags mask
+     * @param value true or false
+     * @return the new flags mask value
+     */
+    private static int set(final int ordinal, final int mask, final boolean value) {
+        return value? mask | 1 << ordinal : mask & ~(1 << ordinal);
     }
+    protected String name;
 
-    public boolean isRedefined() {
-        return isSet(REDEFINED, flags);
-    }
+    protected int symbol = -1;
 
-    public void setShaded(final boolean f) {
-        flags = set(SHADED, flags, f);
-    }
+    protected int flags;
 
-    public boolean isShaded() {
-        return isSet(SHADED, flags);
-    }
-
-    public void setCaptured(final boolean f) {
-        flags = set(CAPTURED, flags, f);
-    }
-
-    public boolean isCaptured() {
-        return isSet(CAPTURED, flags);
-    }
-
-    public boolean isLexical() {
-        return isSet(LEXICAL, flags);
-    }
-
-    public void setLexical(final boolean f) {
-        flags = set(LEXICAL, flags, f);
-    }
-
-    @Override
-    public boolean isConstant() {
-        return isSet(CONST, flags);
-    }
-
-    public void setConstant(final boolean f) {
-        flags = set(CONST, flags, f);
+    ASTIdentifier(final int id) {
+        super(id);
     }
 
     public String getName() {
@@ -138,8 +70,70 @@ public class ASTIdentifier extends JexlNode {
         return null;
     }
 
+    public int getSymbol() {
+        return symbol;
+    }
+
+    public boolean isCaptured() {
+        return isSet(CAPTURED, flags);
+    }
+
+    @Override
+    public boolean isConstant() {
+        return isSet(CONST, flags);
+    }
+
+    public boolean isLexical() {
+        return isSet(LEXICAL, flags);
+    }
+
+    public boolean isRedefined() {
+        return isSet(REDEFINED, flags);
+    }
+
+    public boolean isShaded() {
+        return isSet(SHADED, flags);
+    }
+
     @Override
     public Object jjtAccept(final ParserVisitor visitor, final Object data) {
         return visitor.visit(this, data);
     }
+
+    public void setCaptured(final boolean f) {
+        flags = set(CAPTURED, flags, f);
+    }
+
+    public void setConstant(final boolean f) {
+        flags = set(CONST, flags, f);
+    }
+
+    public void setLexical(final boolean f) {
+        flags = set(LEXICAL, flags, f);
+    }
+
+    public void setRedefined(final boolean f) {
+        flags = set(REDEFINED, flags, f);
+    }
+
+    public void setShaded(final boolean f) {
+        flags = set(SHADED, flags, f);
+    }
+
+    void setSymbol(final int r, final String identifier) {
+        symbol = r;
+        name = identifier;
+    }
+
+    void setSymbol(final String identifier) {
+        if (identifier.charAt(0) == '#') {
+            symbol = Integer.parseInt(identifier.substring(1));
+        }
+        name = identifier;
+    }
+
+    @Override
+    public String toString() {
+        return name;
+    }
 }
