@@ -20,11 +20,14 @@ import static org.apache.commons.jexl3.introspection.JexlPermissions.RESTRICTED;
 import static org.junit.jupiter.api.Assertions.assertArrayEquals;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertInstanceOf;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
 import static org.junit.jupiter.api.Assertions.assertNull;
 import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.junit.jupiter.api.Assertions.fail;
 
+import java.io.Closeable;
 import java.lang.reflect.Method;
 import java.math.BigDecimal;
 import java.util.Arrays;
@@ -36,7 +39,14 @@ import java.util.Map;
 import java.util.Objects;
 import java.util.concurrent.atomic.AtomicLong;
 
+import org.apache.commons.jexl3.internal.Debugger;
+import org.apache.commons.jexl3.internal.Scope;
 import org.apache.commons.jexl3.introspection.JexlPermissions;
+import org.apache.commons.jexl3.parser.ASTJexlScript;
+import org.apache.commons.jexl3.parser.JexlScriptParser;
+import org.apache.commons.jexl3.parser.Parser;
+import org.apache.commons.jexl3.parser.StringProvider;
+import org.junit.jupiter.api.Assertions;
 import org.junit.jupiter.api.Test;
 
 /**
@@ -99,7 +109,7 @@ public class Issues400Test {
     }
 
     @Test
-    public void test402() {
+    void test402() {
         final JexlContext jc = new MapContext();
       // @formatter:off
       final String[] sources = {
@@ -117,7 +127,7 @@ public class Issues400Test {
     }
 
     @Test
-    public void test403() {
+    void test403() {
         // @formatter:off
         final String[] strings = {
             "  map1.`${item.a}` = 1;\n",
@@ -139,17 +149,17 @@ public class Issues400Test {
             final JexlScript script = jexl.createScript(src);
             for (int i = 0; i < 2; ++i) {
                 final Object result = script.execute(null);
-                assertTrue(result instanceof Map);
+                assertInstanceOf(Map.class, result);
                 final Map<?, ?> map = (Map<?, ?>) result;
                 assertEquals(1, map.size());
-                assertTrue(map.containsKey(1));
-                assertTrue(map.containsValue(1));
+                final Object val = jexl.createScript("m -> m[1]").execute(null, map);
+                assertEquals(1, val);
             }
         }
     }
 
     @Test
-    public void test404a() {
+    void test404a() {
         final JexlEngine jexl = new JexlBuilder().cache(64).strict(true).safe(false).create();
         Map<String, Object> a = Collections.singletonMap("b", 42);
         // access is constant
@@ -171,7 +181,7 @@ public class Issues400Test {
     }
 
     @Test
-    public void test404b() {
+    void test404b() {
       // @formatter:off
       final JexlEngine jexl = new JexlBuilder()
           .cache(64)
@@ -212,7 +222,7 @@ public class Issues400Test {
     }
 
     @Test
-    public void test406a() {
+    void test406a() {
         // @formatter:off
         final JexlEngine jexl = new JexlBuilder()
             .cache(64)
@@ -250,7 +260,7 @@ public class Issues400Test {
     }
 
     @Test
-    public void test407() {
+    void test407() {
         // Java version
         final double r = 99.0d + 7.82d - 99.0d - 7.82d;
         assertEquals(0d, r, 8.e-15); // Not zero, IEEE 754
@@ -266,7 +276,7 @@ public class Issues400Test {
     }
 
     @Test
-    public void test412() {
+    void test412() {
         final Map<Object, Object> ctl = new HashMap<>();
         ctl.put("one", 1);
         ctl.put("two", 2);
@@ -284,14 +294,14 @@ public class Issues400Test {
             final JexlEngine jexl = new JexlBuilder().create();
             final JexlScript e = jexl.createScript(fnsrc);
             final Object o = e.execute(jc);
-            assertTrue(o instanceof Map);
+            assertInstanceOf(Map.class, o);
             final Map<?, ?> map = (Map<?, ?>) o;
             assertEquals(map, ctl);
         }
     }
 
     @Test
-    public void test413a() {
+    void test413a() {
         final JexlBuilder builder = new JexlBuilder();
         final JexlEngine jexl = builder.create();
         final JexlScript script = jexl.createScript("var c = 42; var f = y -> c += y; f(z)", "z");
@@ -300,7 +310,7 @@ public class Issues400Test {
     }
 
     @Test
-    public void test413b() {
+    void test413b() {
         final JexlBuilder builder = new JexlBuilder();
         final JexlOptions options = builder.options();
         options.setConstCapture(true);
@@ -312,7 +322,7 @@ public class Issues400Test {
     }
 
     @Test
-    public void test413c() {
+    void test413c() {
         final JexlBuilder builder = new JexlBuilder();
         final JexlEngine jexl = builder.create();
         final JexlScript script = jexl.createScript("#pragma jexl.options '+constCapture'\nvar c = 42; var f = y -> c += y; f(z)", "z");
@@ -321,7 +331,7 @@ public class Issues400Test {
     }
 
     @Test
-    public void test413d() {
+    void test413d() {
         final JexlBuilder builder = new JexlBuilder().features(new JexlFeatures().constCapture(true));
         final JexlEngine jexl = builder.create();
         final JexlException.Parsing xparse = assertThrows(JexlException.Parsing.class, () -> jexl.createScript("var c = 42; var f = y -> c += y; f(z)", "z"),
@@ -330,7 +340,7 @@ public class Issues400Test {
     }
 
     @Test
-    public void test415() {
+    void test415() {
         final JexlBuilder builder = new JexlBuilder().features(new JexlFeatures().constCapture(true));
         final JexlEngine jexl = builder.create();
         JexlScript script;
@@ -353,7 +363,7 @@ public class Issues400Test {
     }
 
     @Test
-    public void test419() throws NoSuchMethodException {
+    void test419() throws NoSuchMethodException {
         // check RESTRICTED permissions denies call to System::currentTimeMillis()
         final Method currentTimeMillis = System.class.getMethod("currentTimeMillis");
         assertFalse(RESTRICTED.allow(currentTimeMillis));
@@ -381,7 +391,7 @@ public class Issues400Test {
     }
 
     @Test
-    public void testDocBreakContinue() {
+    void testDocBreakContinue() {
         final JexlBuilder builder = new JexlBuilder().features(new JexlFeatures().constCapture(true));
         final JexlEngine jexl = builder.create();
         JexlScript script;
@@ -405,7 +415,7 @@ public class Issues400Test {
     }
 
     @Test
-    public void testNamespaceVsTernary0() {
+    void testNamespaceVsTernary0() {
         final VinzContext ctxt = new VinzContext();
         ctxt.set("Users", "USERS");
         final JexlEngine jexl = new JexlBuilder().safe(false).strict(true).silent(false).create();
@@ -424,7 +434,7 @@ public class Issues400Test {
     }
 
     @Test
-    public void testNamespaceVsTernary1() {
+    void testNamespaceVsTernary1() {
         final VinzContext ctxt = new VinzContext();
         ctxt.set("Users", "USERS");
         ctxt.set("vinz", new VinzCaller(ctxt));
@@ -443,4 +453,278 @@ public class Issues400Test {
         assertNotNull(r);
         assertEquals(42, r);
     }
+
+    public static class Ns429 {
+        public int f(final int x) {
+            return x * 10000 + 42;
+        }
+    }
+
+    @Test
+    void test429a() {
+        MapContext ctxt = new MapContext();
+        //ctxt.set("b", 1);
+        JexlFeatures features = JexlFeatures.createDefault();
+        final JexlEngine jexl = new JexlBuilder()
+                .features(features)
+                .safe(false).strict(true).silent(false).create();
+        JexlScript f = jexl.createScript("x -> x");
+        ctxt.set("f", f);
+        String src = "#pragma jexl.namespace.b "+Ns429.class.getName()  +"\n"
+                +"b ? b : f(2);";
+        JexlScript script = jexl.createScript(src, "b");
+        assertEquals(1, (int) script.execute(ctxt, 1));
+
+        src = "#pragma jexl.namespace.b "+Ns429.class.getName()  +"\n"
+                +"b ? b:f(2) : 1;";
+        script = jexl.createScript(src, "b");
+        assertEquals(20042, (int) script.execute(ctxt, 1));
+    }
+
+    @Test
+    void test429b() {
+        MapContext ctxt = new MapContext();
+        ctxt.set("b", 1);
+        JexlFeatures features = JexlFeatures.createDefault();
+        features.namespaceIdentifier(true);
+        final JexlEngine jexl = new JexlBuilder()
+                .features(features)
+                .safe(false).strict(true).silent(false).create();
+        JexlScript f = jexl.createScript("x -> x");
+        ctxt.set("f", f);
+        String src = "#pragma jexl.namespace.b "+Ns429.class.getName()  +"\n"
+                +"b ? b : f(2);";
+        JexlScript script = jexl.createScript(src);
+        assertEquals(1, (int) script.execute(ctxt));
+
+        src = "#pragma jexl.namespace.b "+Ns429.class.getName()  +"\n"
+                +"b ? b:f(2) : 1;";
+        script = jexl.createScript(src);
+        assertEquals(20042, (int) script.execute(ctxt));
+    }
+
+    @Test
+    void test431a() {
+        JexlEngine jexl = new JexlBuilder().create();
+        final String src = "let x = 0; try { x += 19 } catch (let error) { return 169 } try { x += 23 } catch (let error) { return 169 }";
+        final JexlScript script = jexl.createScript(src);
+        assertNotNull(script);
+        final Object result = script.execute(null);
+        assertEquals(42, result);
+    }
+
+    Closeable foo() {
+        return null;
+    }
+
+    @Test
+    void test431b() {
+        JexlEngine jexl = new JexlBuilder().create();
+        final String src = "let x = 0; try(let error) { x += 19 } catch (let error) { return 169 } try { x += 23 } catch (let error) { return 169 }";
+        final JexlScript script = jexl.createScript(src);
+        assertNotNull(script);
+        final Object result = script.execute(null);
+        assertEquals(42, result);
+    }
+
+    @Test
+    void test431c() {
+        JexlEngine jexl = new JexlBuilder().create();
+        final String src = "let xx = 0; try { xx += 19 } catch (let xx) { return 169 }";
+        try {
+            final JexlScript script = jexl.createScript(src);
+            fail("xx is already defined in scope");
+        } catch(JexlException.Parsing parsing) {
+            assertTrue(parsing.getDetail().contains("xx"));
+        }
+    }
+
+    @Test
+    void test433() {
+        JexlEngine jexl = new JexlBuilder().create();
+        final String src = "let condition = true; if (condition) { return; }";
+        final JexlScript script = jexl.createScript(src);
+        assertNotNull(script);
+        Object result = script.execute(null);
+        assertNull(result);
+        Debugger debugger = new Debugger();
+        assertTrue(debugger.debug(script));
+        String dbgStr = debugger.toString();
+        assertTrue(JexlTestCase.equalsIgnoreWhiteSpace(src, dbgStr));
+    }
+
+    @Test
+    void test434() {
+        JexlEngine jexl = new JexlBuilder().safe(false).strict(true).create();
+        final String src = "let foo = null; let value = foo?[bar]";
+        final JexlScript script = jexl.createScript(src);
+        assertNotNull(script);
+        final Object result = script.execute(null);
+        assertNull(result);
+
+    }
+
+    public static class Arithmetic435 extends JexlArithmetic {
+        public Arithmetic435(boolean strict) {
+            super(strict);
+        }
+        public Object empty(String type) {
+            if ("list".equals(type)) {
+                return Collections.emptyList();
+            }
+            return null;
+        }
+    }
+
+    @Test
+    void test435() {
+        JexlArithmetic arithmetic = new Arithmetic435(true);
+        JexlEngine jexl = new JexlBuilder().arithmetic(arithmetic).create();
+        final String src = "empty('list')";
+        final JexlScript script = jexl.createScript(src);
+        assertNotNull(script);
+        final Object result = script.execute(null);
+        assertInstanceOf(List.class, result);
+    }
+
+    @Test
+    void test436a() {
+        String[] srcs = {"let i = null; ++i", "let i; ++i;", "let i; i--;",  "let i; i++;"};
+        run436(null, srcs);
+    }
+
+    @Test
+    void test436b() {
+        String[] srcs = {"var i = null; ++i", "var i; ++i;", "var i; i--;",  "var i; i++;"};
+        run436(null, srcs);
+    }
+
+    @Test
+    void test436c() {
+        JexlContext ctxt = new MapContext();
+        ctxt.set("i", null);
+        String[] srcs = {"++i", "++i;", "i--;",  "i++;"};
+        run436(null, srcs);
+    }
+
+    void run436(JexlContext ctxt, String[] srcs) {
+        final JexlEngine jexl = new JexlBuilder().create();
+        for(String src : srcs) {
+            JexlScript script = jexl.createScript(src);
+            assertThrows(JexlException.Operator.class, () -> script.execute(ctxt));
+        }
+    }
+
+    /** The set of characters that may be followed by a '='.*/
+    static final char[] EQ_FRIEND;
+    static {
+        char[] eq = {'!', ':', '<', '>', '^', '|', '&', '+', '-', '/', '*', '~', '='};
+        Arrays.sort(eq);
+        EQ_FRIEND = eq;
+    }
+
+    /**
+     * Transcodes a SQL-inspired expression to a JEXL expression.
+     * @param expr the expression to transcode
+     * @return the resulting expression
+     */
+    private static String transcodeSQLExpr(final CharSequence expr) {
+        final StringBuilder strb = new StringBuilder(expr.length());
+        final int end = expr.length();
+        char previous = 0;
+        for (int i = 0; i < end; ++i) {
+            char c = expr.charAt(i);
+            if (previous == '<') {
+                // previous char a '<' now followed by '>'
+                if (c == '>') {
+                    // replace '<>' with '!='
+                    strb.append("!=");
+                    previous = c;
+                    continue;
+                } else {
+                    strb.append('<');
+                }
+            }
+            if (c != '<') {
+                if (c == '=') {
+                    // replace '=' with '==' when it does not follow a 'friend'
+                    if (Arrays.binarySearch(EQ_FRIEND, previous) >= 0) {
+                        strb.append(c);
+                    } else {
+                        strb.append("==");
+                    }
+                } else {
+                    strb.append(c);
+                    if (c == '"' || c == '\'') {
+                        // read string, escape '\'
+                        boolean escape = false;
+                        for (i += 1; i < end; ++i) {
+                            final char ec = expr.charAt(i);
+                            strb.append(ec);
+                            if (ec == '\\') {
+                                escape = !escape;
+                            } else if (escape) {
+                                escape = false;
+                            } else if (ec == c) {
+                                break;
+                            }
+                        }
+                    }
+                }
+            }
+            previous = c;
+        }
+        return strb.toString();
+    }
+
+    public static class SQLParser implements JexlScriptParser {
+        final Parser parser;
+
+        public SQLParser() {
+            parser = new Parser(new StringProvider(";"));
+        }
+
+        @Override
+        public ASTJexlScript parse(JexlInfo info, JexlFeatures features, String src, Scope scope) {
+            return parser.parse(info, features, transcodeSQLExpr(src), scope);
+        }
+    }
+
+
+    @Test
+    void testSQLTranspose() {
+        String[] e = { "a<>b", "a = 2", "a.b.c <> '1<>0'" };
+        String[] j = { "a!=b", "a == 2", "a.b.c != '1<>0'" };
+        for(int i = 0; i < e.length; ++i) {
+            String je = transcodeSQLExpr(e[i]);
+            Assertions.assertEquals(j[i], je);
+        }
+    }
+
+    @Test
+    void testSQLNoChange() {
+        String[] e = { "a <= 2", "a >= 2", "a := 2", "a + 3 << 4 > 5",  };
+        for(int i = 0; i < e.length; ++i) {
+            String je = transcodeSQLExpr(e[i]);
+            Assertions.assertEquals(e[i], je);
+        }
+    }
+
+    @Test
+    void test438() {// no local, no lambda, no loops, no-side effects
+        final JexlFeatures f = new JexlFeatures()
+                .localVar(false)
+                .lambda(false)
+                .loops(false)
+                .sideEffect(false)
+                .sideEffectGlobal(false);
+        JexlBuilder builder = new JexlBuilder().parserFactory(SQLParser::new).cache(32).features(f);
+        JexlEngine sqle = builder.create();
+        Assertions.assertTrue((boolean) sqle.createScript("a <> 25", "a").execute(null, 24));
+        Assertions.assertFalse((boolean) sqle.createScript("a <> 25", "a").execute(null, 25));
+        Assertions.assertFalse((boolean) sqle.createScript("a = 25", "a").execute(null, 24));
+        Assertions.assertTrue((boolean) sqle.createScript("a != 25", "a").execute(null, 24));
+        Assertions.assertTrue((boolean) sqle.createScript("a = 25", "a").execute(null, 25));
+        Assertions.assertFalse((boolean) sqle.createScript("a != 25", "a").execute(null, 25));
+    }
 }
