@@ -24,6 +24,7 @@ import static org.junit.jupiter.api.Assertions.fail;
 
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
 
@@ -45,7 +46,7 @@ import org.junit.jupiter.api.Test;
  * Tests sandbox features.
  */
 @SuppressWarnings({ "UnnecessaryBoxing", "AssertEqualsBetweenInconvertibleTypes" })
-public class SandboxTest extends JexlTestCase {
+class SandboxTest extends JexlTestCase {
     public abstract static class AbstractCallMeNot {
         public @NoJexl String NONO = "should not be accessible!";
 
@@ -212,7 +213,7 @@ public class SandboxTest extends JexlTestCase {
     }
 
     @Test
-    public void testCantSeeMe() throws Exception {
+    void testCantSeeMe() {
         final JexlContext jc = new MapContext();
         final String expr = "foo.doIt()";
         JexlScript script;
@@ -232,7 +233,7 @@ public class SandboxTest extends JexlTestCase {
     }
 
     @Test
-    public void testCtorAllow() throws Exception {
+    void testCtorAllow() {
         final String expr = "new('" + Foo.class.getName() + "', '42')";
         JexlScript script;
         Object result;
@@ -247,7 +248,7 @@ public class SandboxTest extends JexlTestCase {
     }
 
     @Test
-    public void testCtorBlock() throws Exception {
+    void testCtorBlock() {
         final String expr = "new('" + Foo.class.getName() + "', '42')";
         final JexlScript script = JEXL.createScript(expr);
         Object result;
@@ -264,7 +265,7 @@ public class SandboxTest extends JexlTestCase {
     }
 
     @Test
-    public void testGetAllow() throws Exception {
+    void testGetAllow() {
         final Foo foo = new Foo("42");
         final String expr = "foo.alias";
         JexlScript script;
@@ -285,7 +286,7 @@ public class SandboxTest extends JexlTestCase {
     }
 
     @Test
-    public void testGetBlock() throws Exception {
+    void testGetBlock() {
         final String expr = "foo.alias";
         final JexlScript script = JEXL.createScript(expr, "foo");
         final Foo foo = new Foo("42");
@@ -302,7 +303,7 @@ public class SandboxTest extends JexlTestCase {
     }
 
     @Test
-    public void testGetNullKeyAllowed0() throws Exception {
+    void testGetNullKeyAllowed0() {
         final JexlEngine jexl = new JexlBuilder().sandbox(new JexlSandbox(true)).create();
         final JexlExpression expression = jexl.createExpression("{null : 'foo'}[null]");
         final Object o = expression.evaluate(null);
@@ -310,7 +311,7 @@ public class SandboxTest extends JexlTestCase {
     }
 
     @Test
-    public void testGetNullKeyAllowed1() throws Exception {
+    void testGetNullKeyAllowed1() {
         final JexlSandbox sandbox = new JexlSandbox(true, true);
         final JexlSandbox.Permissions p = sandbox.permissions("java.util.Map", false, true, true);
         p.read().add("quux");
@@ -329,7 +330,7 @@ public class SandboxTest extends JexlTestCase {
     }
 
     @Test
-    public void testGetNullKeyBlocked() throws Exception {
+    void testGetNullKeyBlocked() {
         final JexlSandbox sandbox = new JexlSandbox(true, true);
         final JexlSandbox.Permissions p = sandbox.permissions("java.util.Map", false, true, true);
         p.read().add(null);
@@ -339,7 +340,7 @@ public class SandboxTest extends JexlTestCase {
         final JexlExpression e0 = jexl.createExpression("{'bar' : 'foo'}['bar']");
         final Object r0 = e0.evaluate(null);
         assertEquals("foo", r0);
-        // can not read quux, null
+        // cannot read quux, null
         for (final String k : Arrays.asList("'quux'", "null")) {
             final JexlExpression expression = jexl.createExpression("{" + k + " : 'foo'}[" + k + "]");
             assertTrue(
@@ -348,27 +349,27 @@ public class SandboxTest extends JexlTestCase {
     }
 
     @Test
-    public void testInheritedPermission0() {
+    void testInheritedPermission0() {
         final Foo386 foo = new Foo386();
         final JexlSandbox sandbox = new JexlSandbox(false, true);
         sandbox.permissions(SomeInterface.class.getName(), true, true, true, true);
         final JexlEngine sjexl = new JexlBuilder().sandbox(sandbox).safe(false).strict(true).create();
         final JexlScript someOp = sjexl.createScript("foo.bar()", "foo");
-        assertEquals(42, someOp.execute(null, foo));
+        assertEquals(42, (int) someOp.execute(null, foo));
     }
 
     @Test
-    public void testInheritedPermission1() {
+    void testInheritedPermission1() {
         final Quux386 foo = new Quux386();
         final JexlSandbox sandbox = new JexlSandbox(false, true);
         sandbox.permissions(Foo386.class.getName(), true, true, true, true);
         final JexlEngine sjexl = new JexlBuilder().sandbox(sandbox).safe(false).strict(true).create();
         final JexlScript someOp = sjexl.createScript("foo.bar()", "foo");
-        assertEquals(-42, someOp.execute(null, foo));
+        assertEquals(-42, (int) someOp.execute(null, foo));
     }
 
     @Test
-    public void testMethodAllow() throws Exception {
+    void testMethodAllow() {
         final Foo foo = new Foo("42");
         final String expr = "foo.Quux()";
         JexlScript script;
@@ -384,7 +385,7 @@ public class SandboxTest extends JexlTestCase {
     }
 
     @Test
-    public void testMethodBlock() throws Exception {
+    void testMethodBlock() {
         final String expr = "foo.Quux()";
         final JexlScript script = JEXL.createScript(expr, "foo");
         final Foo foo = new Foo("42");
@@ -401,7 +402,7 @@ public class SandboxTest extends JexlTestCase {
     }
 
     @Test
-    public void testMethodNoJexl() throws Exception {
+    void testMethodNoJexl() {
         final Foo foo = new Foo("42");
         // @formatter:off
         final String[] exprs = {
@@ -420,7 +421,7 @@ public class SandboxTest extends JexlTestCase {
     }
 
     @Test
-    public void testNoJexl312() throws Exception {
+    void testNoJexl312() {
         final JexlContext ctxt = new MapContext();
         final JexlEngine sjexl = new JexlBuilder().safe(false).strict(true).create();
         final JexlScript foo = sjexl.createScript("x.getFoo()", "x");
@@ -428,7 +429,7 @@ public class SandboxTest extends JexlTestCase {
     }
 
     @Test
-    public void testNonInheritedPermission0() {
+    void testNonInheritedPermission0() {
         final Foo386 foo = new Foo386();
         final JexlSandbox sandbox = new JexlSandbox(false, true);
         sandbox.permissions(SomeInterface.class.getName(), false, true, true, true);
@@ -438,7 +439,7 @@ public class SandboxTest extends JexlTestCase {
     }
 
     @Test
-    public void testNonInheritedPermission1() {
+    void testNonInheritedPermission1() {
         final Quux386 foo = new Quux386();
         final JexlSandbox sandbox = new JexlSandbox(false, true);
         sandbox.permissions(Foo386.class.getName(), false, true, true, true);
@@ -448,13 +449,13 @@ public class SandboxTest extends JexlTestCase {
     }
 
     @Test
-    public void testRestrict() throws Exception {
+    void testRestrict() {
         final JexlContext context = new MapContext();
         context.set("System", System.class);
         final JexlSandbox sandbox = new JexlSandbox();
-        // only allow call to currentTimeMillis (avoid exit, gc, loadLibrary, etc)
+        // only allow call to currentTimeMillis (avoid exit, gc, loadLibrary, etc.)
         sandbox.allow(System.class.getName()).execute("currentTimeMillis");
-        // can not create a new file
+        // cannot create a new file
         sandbox.block(java.io.File.class.getName()).execute("");
         final JexlEngine sjexl = new JexlBuilder().permissions(JexlPermissions.UNRESTRICTED).sandbox(sandbox).safe(false).strict(true).create();
         Object result;
@@ -470,7 +471,7 @@ public class SandboxTest extends JexlTestCase {
     }
 
     @Test
-    public void testSandboxInherit0() throws Exception {
+    void testSandboxInherit0() {
         Object result;
         final JexlContext ctxt = null;
         final List<String> foo = new ArrayList<>();
@@ -483,21 +484,17 @@ public class SandboxTest extends JexlTestCase {
         final JexlScript get = sjexl.createScript("foo[x]", "foo", "x");
 
         result = method.execute(ctxt, foo, "nothing");
-        assertEquals(true, result);
-        result = null;
-        result = get.execute(null, foo, 0);
+        assertTrue((boolean) result);
+        result = get.execute(null, foo, Integer.valueOf(0));
         assertEquals("nothing", result);
-        result = null;
-        result = set.execute(null, foo, 0, "42");
+        result = set.execute(null, foo, Integer.valueOf(0), "42");
         assertEquals("42", result);
-
-        result = null;
-        result = get.execute(null, foo, 0);
+        result = get.execute(null, foo, Integer.valueOf(0));
         assertEquals("42", result);
     }
 
     @Test
-    public void testSandboxInherit1() throws Exception {
+    void testSandboxInherit1() {
         Object result;
         final JexlContext ctxt = null;
         final Operation2 foo = new Operation2(12);
@@ -507,14 +504,14 @@ public class SandboxTest extends JexlTestCase {
         // sandbox.block(Foo.class.getName()).execute();
         final JexlEngine sjexl = new JexlBuilder().sandbox(sandbox).safe(false).strict(true).create();
         final JexlScript someOp = sjexl.createScript("foo.someOp(y)", "foo", "y");
-        result = someOp.execute(ctxt, foo, 30);
-        assertEquals(42, result);
+        result = someOp.execute(ctxt, foo, Integer.valueOf(30));
+        assertEquals(42, (int) result);
         final JexlScript nonCallable = sjexl.createScript("foo.nonCallable(y)", "foo", "y");
-        assertThrows(JexlException.class, () -> nonCallable.execute(null, foo, 0));
+        assertThrows(JexlException.class, () -> nonCallable.execute(null, foo, Integer.valueOf(0)));
     }
 
     @Test
-    public void testSetAllow() throws Exception {
+    void testSetAllow() {
         final Foo foo = new Foo("42");
         final String expr = "foo.alias = $0";
         JexlScript script;
@@ -531,7 +528,7 @@ public class SandboxTest extends JexlTestCase {
     }
 
     @Test
-    public void testSetBlock() throws Exception {
+    void testSetBlock() {
         final String expr = "foo.alias = $0";
         final JexlScript script1 = JEXL.createScript(expr, "foo", "$0");
         final Foo foo = new Foo("42");
@@ -549,7 +546,7 @@ public class SandboxTest extends JexlTestCase {
     }
 
     @Test
-    public void testSetNullKeyAllowed0() throws Exception {
+    void testSetNullKeyAllowed0() {
         final Arithmetic350 a350 = new Arithmetic350(true);
         final JexlEngine jexl = new JexlBuilder().arithmetic(a350).sandbox(new JexlSandbox(true)).create();
         final JexlContext jc = new MapContext();
@@ -560,13 +557,13 @@ public class SandboxTest extends JexlTestCase {
     }
 
     @Test
-    public void testSetNullKeyAllowed1() throws Exception {
+    void testSetNullKeyAllowed1() {
         final Arithmetic350 a350 = new Arithmetic350(true);
         final JexlSandbox sandbox = new JexlSandbox(true, true);
         final JexlSandbox.Permissions p = sandbox.permissions("java.util.Map", true, false, true);
         p.write().add("quux");
         final JexlEngine jexl = new JexlBuilder().arithmetic(a350).sandbox(sandbox).create();
-        // can not write quux
+        // cannot write quux
         final String q = "'quux'"; // quotes are important!
         final JexlExpression expression1 = jexl.createExpression("{" + q + " : 'foo'}[" + q + "] = '42'");
         assertTrue(assertThrows(JexlException.Property.class, () -> expression1.evaluate(null), "should have blocked " + q).getMessage().contains("undefined"));
@@ -583,7 +580,7 @@ public class SandboxTest extends JexlTestCase {
     }
 
     @Test
-    public void testSetNullKeyBlocked() throws Exception {
+    void testSetNullKeyBlocked() {
         final Arithmetic350 a350 = new Arithmetic350(true);
         final JexlSandbox sandbox = new JexlSandbox(true, true);
         final JexlSandbox.Permissions p = sandbox.permissions("java.util.Map", true, false, true);
@@ -595,11 +592,45 @@ public class SandboxTest extends JexlTestCase {
         expression.evaluate(null);
         final Map<?, ?> map = a350.getLastMap();
         assertEquals("42", map.get("bar"));
-        // can not write quux, null
+        // cannot write quux, null
         for (final String k : Arrays.asList("'quux'", "null")) {
             final JexlExpression expression2 = jexl.createExpression("{" + k + " : 'foo'}[" + k + "] = '42'");
             assertTrue(assertThrows(JexlException.Property.class, () -> expression2.evaluate(null), "should have blocked " + k).getMessage()
                     .contains("undefined"));
         }
     }
+    interface I{}
+    static class A implements I{}
+    static class B extends A{}
+
+    @Test
+    void testPermissionOrder() {
+        // permissions should not be dependent on order of evaluation
+        final JexlSandbox sandboxAB = new JexlSandbox(false, true);
+        sandboxAB.permissions(I.class.getName(), true, true, true, false);
+        assertEquals("allow{all}", sandboxAB.get(A.class.getName()).write().toString());
+        assertEquals("allow{all}", sandboxAB.get(B.class.getName()).write().toString());
+        final JexlSandbox sandboxBA = new JexlSandbox(false, true);
+        sandboxBA.permissions(I.class.getName(), true, true, true, false);
+        assertEquals("allow{all}", sandboxBA.get(B.class.getName()).write().toString());
+        assertEquals("allow{all}", sandboxBA.get(A.class.getName()).write().toString());
+    }
+
+    @Test
+    void testIssue424() {
+        final JexlSandbox sandbox = new JexlSandbox(false, true);
+        sandbox.permissions(Map.class.getName(), true, true, true, true);
+        final String jexlCode = "x.foo = 'bar'";
+        final JexlEngine engine = new JexlBuilder()
+                    .sandbox(sandbox)
+                    .safe(false)
+                    .strict(true).create();
+        final JexlContext context = new MapContext();
+        final Map<String, Object> x = new LinkedHashMap<>();
+        context.set("x",  x);
+        final Object result = engine.createScript(jexlCode).execute(context);
+        assertEquals("bar", result);
+        assertEquals("bar", x.get("foo"));
+    }
+
 }
